function V=drahakapek(D)
%V=drahakapek(D)
%V matici "D" obsahujici udaje o detekcich kapek se snazi vystopovat drahy jednotlivych kapek.
%Drahu, ktera ma alespon 10 bodu, vykresli a nabidne ke zpracovani
% - ke zjisteni vertikalni rychlosti kapky prolozenim usecky zavislosti vertikalni pozice na poradi snimku.
%Vznikla matice "V" obsahuje vertikalni rychlosti kapek. Kazdy radek obsahuje rychlosti jedne kapky.
% Liche sloupce jsou vertikalni rychlosti, nasledujici sude sloupce jejich neurcitosti.
% Jednotky rychlosti v matici "V" jsou pixel/snimek.
%
%"D" je matice se zaznamenanymi kapickami. Kazdy sloupec odpovida jedne detekci kapicky.
%  Na prvnim radku je cislo snimku, na kterem byla kapicka zaznamenana,
%  na druhem radku vertikalni pozice kapicky, na tretim horizontalni pozice,
%  na ctvrtem radku je intenzita signalu kapicky.
%
%Pri urcovani, ktere detekce prislusi stejne kapicce, pouziva tato kriteria:
% Signal by se mel idealne objevit na nasledujicim snimku, nejpozdeji na snimku posunutem v poradi o "hcas".
% Vertikalni pozice kapky se muze lisit maximalne o "hvert" pixelu * zpozdeni snimku.
% Horizontalni pozice kapky se muze lisit maximalne o "hhor" pixelu * zpozdeni snimku.
% Relativni intenzita kapky se muze lisit maximalne o "hint".
% Vertikalni pozice kapky se muze lisit maximalne o hextrap*hvert a horizontalni pozice o hextrap*hhor
%  od pozice urcene extrapolaci vychazejici z poslednich (maximalne) peti pozic kapky.
%   (Toto kriterium se nepouzije, kdyz prave doslo k obraceni pole.)
%Kdyz temto kriteriim vyhovuje vice kapek, pak se ta spravna vybira pomoci
% podobnosti intenzity a
% verohodnosti detekovane pozice (vertikalni i horizontalni) odhadovane extrapolaci drahy z poslednich dvou bodu.
%
%Na zacatku vykresli zavislosti poloh kapek na case a zepta se, u kterych snimku doslo k obraceni pole.
% Nasledne nabidne ke zpracovani jen ty kapky, ktere byly zaznamenany pred i po alespon jednom obraceni pole.
%
%Pri vyhodnocovani rychlosti kapky funkce "drahakapek" nejdriv vykresli drahu kapky a pozaduje zadani vektoru "meze".
% Pri prostem odentrovani se tato kapicka ignoruje a jeji draha nevyhodnocuje.
% Obsahuje-li zadany vektor dve cisla, prolozi se drahou kapicky mezi temito dvema poradimi snimku usecka,
%  z jejiz parametru se urci vertikalni rychlost kapicky a jeji neurcitost.
% Po urceni rychlosti se tataz kapicka nabidne k dalsimu zpracovani (k urceni rychlosti jineho useku drahy)
%  a totez se opakuje tak dlouho, dokud misto zadani vektoru "meze" nedojde k prostemu odentrovani.
%Udaje o rychlostech kapicky, jejiz rychlost se spocitala alespon dvakrat, se pridaji do matice "V".


%Parametry urcovani, ktere detekce zaznamenaly stejnou kapku:
hcas=6;
hvert=20;
hhor=5;
hint=0.2;
hextrap=2;

%Vykresleni matice D:
grafkapky(D); xlabel('time  [frames]'); pause(1e-3);

grafkapky(D); xlabel('time  [frames]'); view(0,0); pause(1e-3);
obrac=input('Zadej poradi obrazku, pri kterych se obratilo el. pole. ');
hold on;
for j=1:length(obrac),
 plot3([obrac(j) obrac(j)],[mean(D(3,:)) mean(D(3,:))],[min(D(2,:)) max(D(2,:))],'r');
end


%Vypocet (prirazene detekce se postupne odstranuji z matice "D", dokud matice "D" obsahuje nejake prvky):
V=[];
[nica,m]=size(D);

while m>=1
 K=D(:,1); %Matice "K" bude obsahovat detekce, ktere se prirkly jedne kapce.
 D=D(:,2:end); m=m-1;

 %Vyber casove nejblizsich detekci, ktere by mohly prisluset teze kapce:
 index=-1;
 while length(index)>0,
  dt=1;
  index=[];
  %"index" bude obsahovat poradi sloupcu matice "D" podezrelych z prislusnosti k teze kapce v nasledujicim case.
  % Pozdeji se zredukuje na jedine cislo - poradi nejverohodnejsi detekce teze kapky.
  t=K(1,end); v=K(2,end); h=K(3,end); g=K(4,end);
  while length(index)==0 & dt<=hcas,
   for j=1:m,
    %Pouziti kriteria poradi snimku, podobne pozice a podobne intenzity:
    if (D(1,j)-t)==dt & abs(D(2,j)-v)<=dt*hvert & abs(D(3,j)-h)<=dt*hhor & abs(D(4,j)-g)/g <= hint
     mk=length(K(1,:));
     extrapoluj=0;
     if mk>1
      extrapoluj=2;
      zacatek=max([1 mk-5]);
      for jo=1:length(obrac),
       if D(1,j)>obrac(jo) & K(1,zacatek)<obrac(jo)
        extrapoluj=1;
       end
      end
     end
     if extrapoluj==1 | extrapoluj==2 %Pouziti kriteria podobne ocekavane (extrapolovane) pozice
      zacatek=max([1 mk-5]);
      pr0v=polyfit(K(1,zacatek:mk),K(2,zacatek:mk),1);
      ocekv=polyval(pr0v,K(1,mk)+dt);
      pr0h=polyfit(K(1,zacatek:mk),K(3,zacatek:mk),1);
      ocekh=polyval(pr0h,K(1,mk)+dt);
      if extrapoluj==2
       if abs(D(2,j)-ocekv)<=(hextrap*hvert) & abs(D(3,j)-ocekh)<=hextrap*hhor
        index=[index j];
       end
      elseif extrapoluj==1
       if abs(D(3,j)-ocekh)<=hextrap*hhor
        index=[index j];
       end
      end
     else
      index=[index j];
     end
    end
   end
   dt=dt+1;
  end

  %Vyber nejverohodnejsi detekce, pokud jich je vic:
  if length(index)>1
   %if length(K(1,:))==1
   if extrapoluj==0
    S=(D(4,index)-g).^2;
   %else
   elseif extrapoluj==2
    S1=((D(4,index)-g)/g).^2;
    vocek=v+v-K(2,end-1);
    hocek=h+h-K(3,end-1);
    S2=((D(2,index)-vocek)/hvert).^2;
    S3=((D(3,index)-hocek)/hhor).^2;
    S=S1+S2+S3;
   elseif extrapoluj==1
    S1=((D(4,index)-g)/g).^2;
    %vocek=v+v-K(2,end-1);
    hocek=h+h-K(3,end-1);
    %S2=((D(2,index)-vocek)/hvert).^2;
    S3=((D(3,index)-hocek)/hhor).^2;
    S=S1+S3;
   end
   [nic,por]=min(S);
   index=index(por);
  end

  %Pridani vybrane detekce do matice "K" a jeji odstraneni z matice "D":
  if length(index)>0
   K=[K D(:,index)];
   if index==1
    D=D(:,2:end);
   elseif index==m
    D=D(:,1:(m-1));
   else
    D=D(:,[1:(index-1) (index+1):end]);
   end
   m=m-1;
  end
 end

 %Urceni vertikalnich rychlosti kapky, ktera byla detekovana alespon 10-krat:
 if length(K(1,:))>=10
  vyber=0;
  for jo=1:length(obrac),
   if max(K(1,:))>obrac(jo) & min(K(1,:))<obrac(jo)
    vyber=1;
   end
  end
  if vyber==1
   Fk=figure; plot(K(1,:),K(2,:),'b.'); hold on; grid on;
   xlabel('Poradi snimku'); ylabel('Vertikalni pozice  [pixel]');
   title(['Horizontalni pozice: ' num2str(round(mean(K(3,:))))]); pause(1e-3);
   vv=[]; meze=[0 0];
   while length(meze)==2
    meze=input('Zadej meze pro prolozeni primky: ');
    if length(meze)==2
     [nic,por1]=min(abs(meze(1)-K(1,:)));
     [nic,por2]=min(abs(meze(2)-K(1,:)));
     pr=polyfit(K(1,por1:por2),K(2,por1:por2),1);
     primka=polyval(pr,K(1,por1:por2));
     Soch=sum((K(2,por1:por2)-primka).^2);
     xvar=sum((K(1,por1:por2)).^2) - ((sum(K(1,por1:por2)))^2)/(por2-por1+1);
     vodch=sqrt(Soch/((por2-por1-1)*xvar));
     vv=[vv pr(1) vodch];
     figure(Fk); plot(K(1,por1:por2),primka,'g'); pause(1e-3);
    end
   end
   if length(vv)>=4
    [qa,qb]=size(V);
    V(qa+1,1:length(vv))=vv
   end
  end
 end
end

